#include <stdint.h>
//#include <avr/eeprom.h>
#include "data.h"
#include "servo.h"
#include "distance.h"
#include "command.h"
#include "stepper.h"
#include "i2c_universal.h"

static char *currchar;
static char buf[16];
char iic_received[9]= {5, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa, 0xaa};

void skipspace()
{ while (*currchar == ' ' || *currchar == '\t') 
    currchar++;
}

int getint()
{ int r = 0;
  char sign = 0;
  skipspace();
  if (*currchar == '-')
  { currchar++;
    sign = 1;
  }
  while (*currchar >= '0' && *currchar <= '9')
  { r = 10*r + *currchar - '0';
    currchar++;
  }
  if (sign)
    return -r;
  return r;
}

unsigned gethexnibble()
{ unsigned r = 0;
  if (*currchar >= '0' && *currchar <= '9')
    r = *currchar - '0';
  else if (*currchar >= 'A' && *currchar <= 'F')
    r = *currchar - 'A' + 10;
  else if (*currchar >= 'a' && *currchar <= 'f')
    r = *currchar - 'a' + 10;
  else
    return 0;
  currchar++;
  return r;
}

unsigned gethexbyte()
{ unsigned r = gethexnibble()<<4;
  return r + gethexnibble();
}

unsigned gethexword()
{ unsigned r = gethexbyte()<<8;
  return r + gethexbyte();
}

void handle_command(char idx)
{ static char old_id = -1;
  currchar = msg.body.cmd008.disp.display_msg.text;
  int n, p[4], i;
  static char hex[19];
#ifndef FW_1_24
  if (msg.body.cmd008.disp.display_msg.id == old_id)
    return;
#endif
  old_id = msg.body.cmd008.disp.display_msg.id;
  skipspace();
  switch(*currchar++)
  { case 'e':
    case 'E': n = getint();
	          if (n>0)
			  { thisindex = getint()-1;
                store_index();
              }
              break;
#ifdef USE_SERVO
    case 's':
    case 'S': if (idx != thisindex)
                return;
              n = getint()-1;
			  store_servo(n, getint());
	          break;
#endif
#ifdef USE_DISTANCE
    case 'd':
	case 'D': if (idx != thisindex)
	            return;
	 		  n = getint()-1;
			  for (i = 0; i < 3; i++)
			    p[i] = getint();
              store_distance(n, p);
	          break;
    case 'r':
	case 'R': raw_distance = getint();
	          break;
#endif
#ifdef USE_STEPPER
    case 'm':
	case 'M': //motor parameters <motor> <motpars_stallpars> <abs>
	          if (idx != thisindex) return;
	 		  n = getint()-1;
			  if (n < 0) return;
			  skipspace();
			  for (i=0; i < 18; i++)
			    hex[i] = gethexbyte();
              hex[18] = getint();
              set_stepper_pars(n, hex);
			  break;
#endif
#ifdef USE_I2C
    case 'I': //I2C write/read command <nrofbytestoread> <hexaddr> <hexwritedata>...
	          n = getint(); //nr of bytes to read
	          for ( i = 1; *currchar && i < 16; i++) 
			  { skipspace();
			    buf[i] = gethexbyte(); //the i2c address followed by the data to write
              }
			  buf[0] = i-1; //the number of bytes to write
	          while (i2c_busy()) /* do nothing */ ;
			  if (n==0)
			    i2c_write(buf);
              else
			  { iic_received[0] = n;
			    i2c_read(buf, iic_received);
              }
	          break;
    case 'B': //set the bytemap
	          for ( i = 0; *currchar && i < 4; i++) 
			    bytemap[i] = getint(); //the indices in the 'iic_received' array of bytes to report in the button fields
	          break;
#endif
#ifdef USE_FWUPGRADE
    case ':'  //intel hex record
	          p[0] = gethexbyte(); //nr of bytes in this record
			  if (p[0] > 16) return;
			  p[1] = gethexword(); //address
			  p[2] = gethexbyte(); //type
			  for (i = 0; *currpos && i < p[0]; i++)
			     buf[i] = gethexbyte();
              p[3] = gethexbyte(); //checksum
	          break;
#endif
    default:
	          break;
  }
}
