#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "servo.h"
#include "data.h"

//use timer0
#define SERVOP	PORTC
#define SERVOD	DDRC
#define SERVOB	0
#define SERVOM	(((1<<NSERVOS)-1)<<SERVOB)

#ifdef USE_SERVO

short int servopos[NSERVOS];
char servorange[NSERVOS];
char ee_servorange[NSERVOS] __attribute__((section(".eeprom"))) = {2,2,2,2};

static short int servo = 0;
static char enabled;
  
void init_servos()
{ //init timer
  TCCR0A = 0; //normal mode, no outputs
#if  F_CPU == 7372800
// clk/64=>115200Hz ~ 8.7us,  range = 0-2.22ms, 0.5-1.5 corresponds to 115 counts (57-172)
  TCCR0B = 3; //prescaler=64
#elif F_CPU == 14745600
// clk/256=>57600Hz ~ 17.4us,  range = 0-4.44ms, 0.5-1.5 corresponds to 57 counts (28-86)
  TCCR0B = 4; //prescaler=256
#else
#error "define divider for cpu freq "  F_CPU
#endif
  TIMSK0 = _BV(OCIE0A); //interrupt on output compare A match
  //set outputs low
  SERVOP &= ~SERVOM; //data
  SERVOD |= SERVOM; //direction
  eeprom_read_block(servorange, ee_servorange, sizeof(servorange));
}

void set_servos()
{ short int i;
  for (i = 0; i < NSERVOS; i++)
    servopos[i] = msg.body.cmd002.output.duty[i]; //assume duty range is 0-511
}

void enable_servos(char m)
{ m &= SERVOM;
  SERVOD = (SERVOD & ~SERVOM) | m;
  enabled = m;
}

void store_servo(unsigned char n, char v)
{ if (eeprom_busy)
    return;
  n--;
  if (n < NSERVOS)
    eeprom_write_byte(&servorange[n], v);
}

void update_servos()
{ //set pulsewidth for current servo, set output high and start timer
  servo++;
  if (servo >= NSERVOS)
    servo = 0;
  cli(); //eliminate startpoint as source of jitter
  SERVOP |= _BV(SERVOB+servo) & enabled; //set bit
  TCNT0 = 0; //reset counter
  sei();
#if  F_CPU == 7372800
  OCR0A = (servopos[servo]>>2) + 58; //to get in range of 0.49ms - 1.58ms
#elif F_CPU == 14745600
  switch (servorange[servo])
  { case 0: OCR0A = (servopos[servo]>>2) + 20; //to get in range of 0.35ms - 2.56ms, suitable for modelcraft mini servo
            break;
	case 1: OCR0A = (servopos[servo]>>3) + 54; //to get in range of 0.95ms - 2.04ms 
	        break;
    case 2: OCR0A = (servopos[servo]>>4) + 71; //to get in range of 1.24ms - 1.77ms, suitable for controlset servo
	        break;
    default:OCR0A = (servopos[servo]>>4) + 71; //to get in range of 1.24ms - 1.77ms, by default use smallest range
	        break;
  }
#else
#error "define divider for cpu freq "  F_CPU
#endif
}

#if 0
ISR(TIMER0_COMPA_vect) //invoked when pulsewidth timer expires
{ //set output of servo low again
  SERVOP &= ~SERVOM;
}
#else
ISR(TIMER0_COMPA_vect, ISR_NAKED) //invoked when pulsewidth timer expires, jitter occurs here due to non-constant latency
{ //set output of servo low again
  int i;
  for (i = 0; i < NSERVOS; i++) //compiler unwinds this loop
    asm ("cbi %0,%1"::"I" (_SFR_IO_ADDR(SERVOP)), "I" (SERVOB+i));
  reti();
}
#endif

#endif
