#include <avr/io.h>
//#include <avr/interrupt.h>
#include <avr/eeprom.h>
//#include <avr/pgmspace.h>
#include <stdlib.h>

#define DEBUG
#include "data.h"



#ifdef USE_STEPPER
#include "stepper.h"
#include "i2c_universal.h"

stepper_desc steppers[4];
/****************************************************************************************************************/
#ifdef MCP23017
#define DEVADR 0x40

//these initializers must be updated
char cmdbuf[] = {4, DEVADR, 0, 0x3f, 0xfc }; //set some pins as output

static const char outputs[4][8] = {{0x17, 0x05, 0x27, 0x22, 0x2B, 0x09, 0x1B, 0x12}, //bits are: x x A B C D E1 E2
                                   {0x5C, 0x14, 0x9C, 0x88, 0xAC, 0x24, 0x6C, 0x48}, //bits are: A B C D E1 E2 x x 
								   {0x17, 0x05, 0x27, 0x22, 0x2B, 0x09, 0x1B, 0x12},
                                   {0x5C, 0x14, 0x9C, 0x88, 0xAC, 0x24, 0x6C, 0x48}
                                  };

static char state_machine(int n)
{ static char state[4];
  int dir = (steppers[n].speed<0) ? -1: 1;
  switch (steppers[n].mode)
  { case 0: //full step
            if (state[n] & 1) //odd
			  state[n] += dir;
            else
			  state[n] += dir<<1;
			break;
    case 1: //half step
	        state[n] += dir;
			break;
    case 2: //wave drive
	        if (state[n] & 1) //odd
			  state[n] += dir<<1;
            else            
			  state[n] += dir;
  }
  state[n] &= 7;
  return outputs[n][state[n]];
}

static void update_stepper(int i)
{ //update stepper state machine
   if (steppers[i].speed != 0)//do nothing when speed == 0
   { if (--steppers[i].count == 0)
     { if (steppers[i].speed < 0) //alternatively use reciprocal: count = 511/speed
         steppers[i].count = steppers[i].speed + 512;
       else
         steppers[i].count = 512 - steppers[i].speed;
       if (steppers[i].pos < steppers[i].dest)
	   { char val = state_machine(i);
	     steppers[i].pos++;
		 switch(i)
		 { case 0: cmdbuf[1] = DEVADR;
                   cmdbuf[2] = 0x14; //OLATA
				   cmdbuf[3] = val;
				   break;
           case 1: cmdbuf[4] = val;
		           i2c_write(cmdbuf);
				   break;
		   case 2: cmdbuf[1] = DEVADR+1;
                   cmdbuf[2] = 0x14; //OLATA
				   cmdbuf[3] = val;
				   break;
           case 3: cmdbuf[4] = val;
		           i2c_write(cmdbuf);
				   break;
         }
       }
     }
   }
}

void update_steppers()
{ update_stepper(0);
  update_stepper(1);
}

void init_stepper()
{ int i;
  IIC1_Init();
  i2c_write(cmdbuf);
  while (i2c_busy()) /* do nothing */ ;
  cmdbuf[1] = DEVADR+1;
  i2c_write(cmdbuf);
  for (i = 0; i < 4; i++)
  { steppers[i].mode = 1; //get this from eeprom and/or command line
    steppers[i].count = 1; //immediate setting from speed
  }
}

void set_stepper(int i, int dest, int speed, bool reset)
{ steppers[i].dest = dest;
  steppers[i].speed = speed;
  if (reset)
    steppers[i].pos = 0;
}


#endif //MCP23017

/*********************************************************************************************************/

#ifdef	AVR_STEPPER

#define DEVADR 0x40

//these initializers must be updated
char cmdbuf[] = {4, DEVADR, 0, 0x3f, 0xfc }; //define step mode

void update_steppers()
{ //check if stepper reached destination and copy to TA
  //copy steppers[].dest and steppers[].speed to I2C slave
  //read steppers[].pos from I2C slave
}

void init_stepper()
{ int i;
  IIC1_Init();
  i2c_write(cmdbuf);
  while (i2c_busy()) /* do nothing */ ;
  cmdbuf[1] = DEVADR+1;
  i2c_write(cmdbuf);
  for (i = 0; i < 4; i++)
  { steppers[i].mode = 1; //get this from eeprom and/or command line
    steppers[i].count = 1; //immediate setting from speed
  }
}

#endif //AVR_STEPPER

/**************************************************************************************************************/

#ifdef AMIS30624


//stepper motor flags
#define START	0x01  	//set when advanced motor control is started
#define RESET	0x02  	//set when cnt_reset changes
#define SPEED	0x04	//set when speed changes, cleared on setMotPar
#define DEST	0x08	//set when destination changes
#define TIME	0x10	//set by timerint when it is time to poll status
#define ABS		0x20	//fixed bit, when set positioning is absolute, when clear positioning is relative
#define DECR	0x40	//set when ack_reset meeds to be decremented, cleared on next INPUT/OUTPUT
#define SYNC	0x80	//set when the motor is synchronised to another motor

//#define STEP	1		//stepsize for bresenham, currently all gridpoints must be a multiple of the stepsize
//#define STEP	(1<<((motpar[i][8]&0xC0)>>2))		//stepsize for bresenham, currently all gridpoints must be a multiple of the stepsize
//#define STEP	(steps[i])

char devadr[4] = { 0xC0, 0xC2, 0xC4, 0xC6}; //i2c device addresses of amis controllers

const struct _pars
{ char motpar[9];
  char stallpar[9];
  char absolute;
} parameters[4] EEMEM = {{{ 0xC0, 0x89, 0xFF, 0xFF, 0xBF, 0x5F, 0x02, 0, 0xAE},{ 0xC0, 0x96, 0xFF, 0xFF, 0xBF, 0x5F, 0x02, 0, 0x0C}, 1},
                         {{ 0xC2, 0x89, 0xFF, 0xFF, 0xBF, 0x5F, 0x02, 0, 0xAE},{ 0xC2, 0x96, 0xFF, 0xFF, 0xBF, 0x5F, 0x02, 0, 0x0C}, 1},
						 {{ 0xC4, 0x89, 0xFF, 0xFF, 0xBF, 0x5F, 0x02, 0, 0xAE},{ 0xC4, 0x96, 0xFF, 0xFF, 0xBF, 0x5F, 0x02, 0, 0x0C}, 1},
						 {{ 0xC6, 0x89, 0xFF, 0xFF, 0xBF, 0x5F, 0x02, 0, 0xAE},{ 0xC6, 0x96, 0xFF, 0xFF, 0xBF, 0x5F, 0x02, 0, 0x0C}, 1}};
char cmdbuf[10];
char statbuf[9];
int steps[4];

static int getStatus1(int i)
{ if (devadr[i] == 0xFF) return I2C_NODEV;
  cmdbuf[0] = 2;
  cmdbuf[1] = devadr[i];
  cmdbuf[2] = 0x81; //getfullstatus1
  statbuf[0] = 8; //read 8 bytes
  return i2c_read(cmdbuf, statbuf);
}

static int getStatus2(int i)
{ if (devadr[i] == 0xFF) return I2C_NODEV;
  cmdbuf[0] = 2;
  cmdbuf[1] = devadr[i];
  cmdbuf[2] = 0xFC; //getfullstatus2
  statbuf[0] = 8; //read 8 bytes
  return i2c_read(cmdbuf, statbuf);
}

static void getMotPar(int i)
{ cmdbuf[0] = 9;
  eeprom_read_block(cmdbuf+1, parameters[i].motpar, 9);
}

static int setMotPar(int i)
{ if (devadr[i] == 0xFF) return I2C_NODEV;
  //cmdbuf[0] = 9;
  //eeprom_read_block(cmdbuf+1, &motpar[i][0], 9);
  //cmdbuf[1] = devadr[i];
  return i2c_write(cmdbuf);
}

static int setStallPar(int i)
{ if (devadr[i] == 0xFF) return I2C_NODEV;
  cmdbuf[0] = 9;
  eeprom_read_block(cmdbuf+1, parameters[i].stallpar, 9);
  //cmdbuf[1] = devadr[i];
  return i2c_write(cmdbuf);
}

static int resetPos(int i)
{ if (devadr[i] == 0xFF) return I2C_NODEV;
  steppers[i].pos = 0;
  cmdbuf[0] = 2;
  cmdbuf[1] = devadr[i];
  cmdbuf[2] = 0x86;
  return i2c_write(cmdbuf);
}

static int softStop(int i)
{ if (devadr[i] == 0xFF) return I2C_NODEV;
  cmdbuf[0] = 2;
  cmdbuf[1] = devadr[i];
  cmdbuf[2] = 0x8F;
  return i2c_write(cmdbuf);
}

static int setPos(int i, int n)
{ if (devadr[i] == 0xFF) return I2C_NODEV;
  cmdbuf[0] = 6;
  cmdbuf[1] = devadr[i];
  cmdbuf[2] = 0x8B;
  cmdbuf[3] = 0xFF;
  cmdbuf[4] = 0xFF;
  cmdbuf[5] = n>>8;
  cmdbuf[6] = n;
  return i2c_write(cmdbuf);
}

void update_steppers()//this is called periodically by the timer interrupt
{ static char mot = 0;
  if (steppers[mot].state == setpos)
  {
    steppers[mot].flags |= TIME;
  }
  mot++;
  mot &= 3;
}

static void init_stepper(int i)
{ setStallPar(i); //copy eeprom stall parameters to AMIS
  while (i2c_busy()) /* do nothing */ ;
  if (i2clasterror()) //if fail, mark device as absent
    devadr[i] = 0xFF;
  else
  { int j;
    devadr[i] = cmdbuf[1]; //copy the actual device address into ram
    getMotPar(i);          //get motor parameters from eeprom
    steps[i] = 8>>((cmdbuf[9]&0x0C)>>2);  //get the stepsize for bresenham
    setMotPar(i);							//copy the parametrs to the AMIS
    while (i2c_busy()) /* do nothing */ ;
    for (j=0; j<3; j++) //retry ...
    { char err;
      getStatus1(i);
      while (i2c_busy()) /* do nothing */ ;
      err = statbuf[5]; //any problem?
	  statbuf[0] = 0; //release buffer 
	  if (err==0)//no electrical or thermal errors
	    break; 
    }
	if (j>=3)
	  devadr[i] = 0xFF; //disable the motor if error persists
    getStatus2(i);
    while (i2c_busy()) /* do nothing */ ;
    steppers[i].pos = (statbuf[2]<<8)|statbuf[3];//actual position
	statbuf[0] = 0; //release buffer 
  }
  steppers[i].state = idle;
  steppers[i].flags = eeprom_read_byte(&parameters[i].absolute) ? ABS : 0; 
}

void init_steppers()//called after reset of the microcontroller
{ int i;
  for (i = 0; i < 4; i++)
    init_stepper(i);
}

void set_stepper_pars(char n, char p[])//called in command.c to copy user parameters to eeprom
{ if (n>3) return;
  eeprom_write_block_nonblocking(p, (char*)&parameters[n], sizeof(struct _pars));
  steppers[n].state = initmot;
}

void set_stepper(TA_OUTPUT *out)//called when an output message cmd002 is received
{ int i, j=0;
  for (i = 0; i < 4; i++) //first set all SYNC flags false and later set them when the motors are supposed to move in sync
  { //steppers[i].master = 0;
    steppers[i].flags &= ~SYNC;
  }
  for (i = 0; i < 4; i++)
  { if (steppers[i].flags & DECR) //flag to indicate that the counter reset must be acknowledged
    { steppers[i].flags &= ~DECR;
      steppers[i].ack_reset--;
    }
    if (steppers[i].starts != out->reserved[i]) //a new advanced motor control command was received (i.e. the distance has changed)
    { steppers[i].flags |= START|DECR;
      steppers[i].starts = out->reserved[i]; 
	  //steppers[i].motor_pos_reached = 0;//detected by change not by value
	  steppers[i].ack_reset++;
    }
    int speed = out->duty[j++];
	speed -= out->duty[j++]; //compute the speed as the difference in duty cycles
    if (steppers[i].speed != speed)//the speed has changed (including direction, stop and start
    { steppers[i].flags |= SPEED;
      steppers[i].speed = speed; 
    }
	int dest = out->distance[i];
	dest &= ~(steps[i]-1); //round down to nearest multiple of stepsize
    if (steppers[i].dest != dest)//the distance has changed
    { steppers[i].flags |= DEST;
      steppers[i].dest = dest;
    }
    if (steppers[i].resets != out->cnt_reset[i])//the counter must be reset
    { steppers[i].flags |= RESET;
      steppers[i].resets = out->cnt_reset[i]; 
      steppers[i].ack_reset++; //acknowledge the reset
    } 
	int m = out->master[i];
	steppers[i].master = m; //indicate which motor (m-1) is the master for this slave (i)
	if (m != 0)
	{ steppers[i].flags |= SYNC;   //set the SYNC for both the master and slave
	  steppers[m-1].flags |= SYNC;
	} 
  }
}

static void setPosition(int i)
{ if (!(steppers[i].flags & SYNC))
    setPos(i, steppers[i].dest); //normal case
  else //bresenham case
  { int delta = steppers[i].dest - steppers[i].pos;//initial bresenham calculation for one of the two motors
    if (delta > 0)
	{ steppers[i].delta = delta;
	  steppers[i].step = steps[i];
    }
	else
	{ steppers[i].delta = -delta;
	  steppers[i].step = -steps[i];
    }
	if (steppers[i].master != 0) // slave
	{ int m = steppers[i].master-1;
	  //setPosition(m); //recursively call setPosition for the master to make sure that delta is properly defined
	  int deltam = abs(steppers[m].dest - steppers[m].pos);
      if ((steppers[i].steep = (steppers[i].delta > deltam)))//compute 'steep' (for slave only), true when slave movement is larger than master movement
	  //compute the initial error (for slave only), depending on 'steep'
	    steppers[i].eps = -steppers[i].delta/2; //slave is 'y' direction
	  else
	    steppers[i].eps = deltam/2; //master = 'x' direction
      CLRPIN11;
	}
	else
	{ steppers[i].m_rdy = 1;
      CLRPIN12;
	}
    LEDOFF;
  }//don't call setPos, let runslice determine which move to make
}

//call this function only for sync motors (1 master and 1 slave), and when actpos==tagpos for this motor
//the run slice function can return for several reasons:
// - the motor is a master
// - the master is not ready (has not finished the previous command)
// - a new setPos command was issued
// if the motor is a master, it must return to the setpos state
// if a new command was issued, it must return to the setpos state
static char runslice(int i)//this is the flesh and bones of the bresenham algorithm
{ if (steppers[i].master==0)
  { steppers[i].m_rdy = 1;//signal to the slave that the master is ready
    CLRPIN12;//master done
    return 0;//skip the master motor but service the master together with the slave
  }
  CLRPIN11;//slave done
  int m = steppers[i].master - 1;
  if (!steppers[m].m_rdy)
    return 2; //master not ready yet
  int y = steppers[i].pos; //x.y represents the actual position of the motors
  int x = steppers[m].pos;
  if (y==steppers[i].dest && x==steppers[m].dest) return 3; //this test should not be necessary, just to find a bug
  if (steppers[i].steep) //coordinates are swapped for steep lines
  { y += steppers[i].step; //y is dominant direction so we take an y-step in any case
	steppers[i].eps += steppers[m].delta;
    if (steppers[i].eps > 0) //diagonal step
	{ steppers[i].eps -= steppers[i].delta;
	  x += steppers[m].step; //also take a y-step
	  steppers[m].m_rdy = 0;
      SETPIN12;
	  setPos(m, x);
    }
    else
      while (steppers[i].eps <= -steppers[m].delta && y != steppers[i].dest) 
      { steppers[i].eps += steppers[m].delta;        
	    y += steppers[i].step; 
	  }
    SETPIN11;
    while (i2c_busy()) /* do nothing */ ; //busy waiting, ugly but position commands will be back to back
    setPos(i, y); //issue the command to the motor to make all necessary y-steps
  }
  else
  { x += steppers[m].step;
	steppers[i].eps -= steppers[i].delta;
    if (steppers[i].eps < 0) //diagonal run
	{ steppers[i].eps += steppers[m].delta;
      y += steppers[i].step;
      SETPIN11;
      setPos(i, y);
    }
	else
   	  while (steppers[i].eps >= steppers[i].delta && x != steppers[m].dest)
	  { steppers[i].eps -= steppers[i].delta;
	    x += steppers[m].step;
	  }
    steppers[m].m_rdy = 0;
    SETPIN12;
    while (i2c_busy()) /* do nothing */ ;
    setPos(m, x);
  }
  return 1;
}

static bool test_end(int i, int pos)//test if the move or line is finished
{ if (pos != steppers[i].dest) return false; //destination not reached yet
  //destination reached for mot i
  if (!(steppers[i].flags & SYNC)) return true; //mot i is not synchronous, so the position was reached
  //synchronous motor i reached position
  if (steppers[i].master == 0) return false; //always return false for the master, wait for the slave
  //slave reached position
  LEDON;
  int m = steppers[i].master-1; //find the master
  if (steppers[m].pos == steppers[m].dest) //the master also reached its destination
  { steppers[m].motor_pos_reached = steppers[m].starts;//signal to host that master is ready
    steppers[m].state = idle;
	CLRPIN12;
	return true; //both motors reached their destination
  }
  CLRPIN11;
  return false;//waiting for the master to finish
}

//#error there appears to be a problem with changing direction (speed), furthermore bresenham (runslice, testend) is completely untested
//this represents 4 statemachines ( 1 for each motor) there is however only one cmd buffer and 1 status buffer
//and of course only one I2C interface. The I2C interrupt is concurrent with these statemachines. The state machines
//are only concurrent among each other for reads. It is not possible that two state machines issue a command at the same time
//or even use the cmd buffer at the same time. This is because the first test is for i2c_busy, this implies that previous commans
//has finished, the setup and submission of the new buffer is atomic w.r.t. the other statemachines.
//for reads there is the further condition that the statbuffer has to be free before issuing the command.
#if 1
void setSpeed(char i)
{ getMotPar(i);
  cmdbuf[6] &= 0x0F;//only set Vmax, theoretically 9 bit + direction are available
  if (steppers[i].speed > 0)
  { cmdbuf[6] |= ((steppers[i].speed-1)<<4);//map 1-16 onto 0-15 and shift into high nibble
    cmdbuf[7] &= 0xEF; //clear shaft bit
  }
  else //speed < 0
  { cmdbuf[6] |= ((-steppers[i].speed-1)<<4);//map -1 - -16 onto 0-15 and shift into high nibble
    cmdbuf[7] |= 0x10; //set shaft bit
  }
  setMotPar(i);
}

void poll_stepper() //put this in the main loop
{  int i;
   for (i=0; i < 4; i++)
   {  if (i2c_busy()) return;
      switch (steppers[i].state)
	  { case idle:      if(steppers[i].flags & RESET)
				    	{ steppers[i].flags &= ~RESET;
				     	  resetPos(i);
				   		} 
						else if (steppers[i].flags & SPEED)
	               		{ steppers[i].flags &= ~SPEED;
						  if (steppers[i].speed != 0) 
						    setSpeed(i);
                        }
				   		else if((steppers[i].flags & START) && steppers[i].speed != 0)
				   		{ steppers[i].state = setpos;
					   	  steppers[i].flags &= ~START;
					   	  setPosition(i);
					 	}
				   		break;
        case setpos: //motor is running, regularly poll status for position and problems
		                //the AMIS ignores resetPosition while moving, hence the missing transition
					   if (steppers[i].flags & SPEED) //speed was changed
					   { steppers[i].flags &= ~SPEED;
                         if (steppers[i].speed == 0) //motor stop?
						 { softStop(i);						//stop the motor, actpos ==> tagpos
					       steppers[i].state = idle;//need SPEED and START to get out of idle state, always need setPosition and usually setSpeed
                         }
						 else 
				           setSpeed(i);//careful with conditions (stay in speedgroup).
                       }
					   else if (steppers[i].flags & START)
					   { setPosition(i);              //accept a (new) distance
						 steppers[i].flags &= ~START;
                       }
                       else if ((steppers[i].flags & (TIME|SYNC)) && statbuf[0]==0)//time to read status and status buffer is free
					   { steppers[i].state = status;
					     steppers[i].flags &= ~TIME;
					     getStatus2(i); //read the position 	  
					   }
				       break;
        case status:   { int pos = (statbuf[2]<<8)|statbuf[3];//actual position
					     int tgt = (statbuf[4]<<8)|statbuf[5];//target position
                         steppers[i].pos = pos; //report actual position back to master
						 statbuf[0] = 0;//release the buffer
						 if (test_end(i, pos))//check the stop condition for synchronous and non-synchronous mode
						 { if (!(steppers[i].flags & ABS))
				     	     resetPos(i);
						   steppers[i].state = idle;
						   steppers[i].motor_pos_reached = steppers[i].starts;//signal to host
                         }
						 else if (pos == tgt && (steppers[i].flags & SYNC))//target reached but not the final destination. Condition also occurs on hard/softstop
						                                                   //hence extra condition for sync-motors 
						 { char res = runslice(i);//compute new line segment
						   //res==0 means 'master', res==1 means 'new position', res==2 means 'master not ready', res==3 means 'destination reached'
						   steppers[i].state = setpos;
						 } 
						 else //if (steppers[i].flags & TIME)
						 { steppers[i].state = status1;
						   getStatus1(i);//get ESW and error data
                         } 
					    // steppers[i].flags &= ~TIME;
					   }
					   break;
        case status1:  { char err = statbuf[5];
					     char stall = statbuf[6] & 0x02;
					     steppers[i].esw = (statbuf[6] & 0x10) != 0;
						 statbuf[0] = 0; //release buffer
						 if (err & 0xB8) //ignore thermal warnings and steploss
						 { //SETPIN11; //motor is probably in shutdown
						   steppers[i].state = idle;
						   steppers[i].pos = steppers[i].dest;
						   steppers[i].motor_pos_reached = steppers[i].starts;//signal to host
                         } 
						 else
						    steppers[i].state = setpos;
					   }
					   break;
        //case stop:	   break;
        case initmot:  if (!eeprom_busy)
		                  init_stepper(i);
					   break;
	  }
   }
}
#else
void poll_stepper() //put this in the main loop
{  int i;
   for (i=0; i < 4; i++)
      switch (steppers[i].state)
	  { case idle: if (i2c_busy()) break;
	               if (steppers[i].flags & SPEED) //any change in speed
	               { if (steppers[i].speed != 0) //not stop
				     { steppers[i].state = setparam;
					   steppers[i].flags &= ~SPEED;
					   getMotPar(i);
					   cmdbuf[6] &= 0x0F;//only set Vmax, theoretically 9 bit + direction are available
					   if (steppers[i].speed > 0)
					   { cmdbuf[6] |= ((steppers[i].speed-1)<<4);//map 1-16 onto 0-15 and shift into high nibble
					     cmdbuf[7] &= 0xEF; //clear shaft bit
                       }
					   else //speed < 0
					   { cmdbuf[6] |= ((-steppers[i].speed-1)<<4);//map -1 - -16 onto 0-15 and shift into high nibble
					     cmdbuf[7] |= 0x10; //set shaft bit
                       }
					   setMotPar(i);
					 } //else speed is 0, stay idle
				   } 
				   else if(steppers[i].flags & START)
				   { if (steppers[i].flags & ABS)
				     { steppers[i].state = setpos;
					   steppers[i].flags &= ~START;
					   setPosition(i);
					 }
					 else
					 { steppers[i].state = resetpos;
					   resetPos(i);
					 }
                   }	
				   else if(steppers[i].flags & RESET)
				   { steppers[i].state = resetpos;
				     resetPos(i);
				   }
				   break;
        case setparam: if (i2c_busy()) break;
		               if (steppers[i].flags & START)
					   { if (steppers[i].flags & ABS)
						 { steppers[i].state = setpos;
					       setPosition(i);
						   steppers[i].flags &= ~START;
						 }
						 else
		                 { steppers[i].state = resetpos;
					       resetPos(i);
					     }
                       }
                       break;
        case resetpos: if (i2c_busy()) break;
		               if (1)
		               { steppers[i].state = setpos;
					     setPosition(i); //set tagpos to distance (actpos==zero, so restart the movement)
						 steppers[i].flags &= ~(START|RESET);
					   }
					   break;
        case setpos: //motor is running, regularly poll status for position and problems
		               if (i2c_busy()) break;
		               if (steppers[i].flags & RESET)
		               { steppers[i].state = resetpos;
					     resetPos(i); //set actpos and tagpos to zero, ignored when motor is running
						 //with real TX and encoder motor, the counter is reset to zero so the motor keeps on running
						 //this can only be simulated by changing tagpos but then actpos does not correspond directly to the counter
					   }
					   else if (steppers[i].flags & START)
					   { if (steppers[i].flags & ABS)
						 { setPosition(i);              //accept a (new) distance
						   steppers[i].flags &= ~START;
						 }
						 else
		                 { steppers[i].state = resetpos;
					       resetPos(i);//same issue, positions cannot be reset on the fly
					     }
                       }
                       else if (steppers[i].flags & SPEED) //speed was changed
					   { if (steppers[i].speed == 0) //motor stop?
						 { softStop(i);						//stop the motor, actpos ==> tagpos
					       steppers[i].state = stop;
					       steppers[i].flags &= ~SPEED;
                         }
						 else 
				         { steppers[i].state = speed; //useless, may as well stay in setpos
					       steppers[i].flags &= ~SPEED;
					       getMotPar(i);
					       cmdbuf[6] &= 0x0F;//only set Vmax, theoretically 9 bit + direction are available
					       if (steppers[i].speed > 0)
					       { cmdbuf[6] |= ((steppers[i].speed-1)<<4);//map 1-16 onto 0-15 and shift into high nibble
					         cmdbuf[7] &= 0xEF; //clear shaft bit
                           }
					       else
					       { cmdbuf[6] |= ((-steppers[i].speed-1)<<4);//map -1 - -16 onto 0-15 and shift into high nibble
					         cmdbuf[7] |= 0x10; //set shaft bit
                           }
					       setMotPar(i);
					     }
                       }
					   else if ((steppers[i].flags & TIME) && statbuf[0]==0)//time to read status and status buffer is free
					   { steppers[i].state = status;
					     steppers[i].flags &= ~TIME;
						 CLRPIN11;
					     getStatus2(i); //read the position 	  
					   }
				       break;
        case status:   if (i2c_busy()) break; //status read (or other i2c activity) still busy
		               //if (statbuf[0] == 0) //the received message is complete
					   if (1)
		               { int pos = (statbuf[2]<<8)|statbuf[3];//actual position
					     int tgt = (statbuf[4]<<8)|statbuf[5];//target position
                         steppers[i].pos = pos; //report actual position back to master
						 statbuf[0] = 0;//release the buffer
						 //steppers[i].state &= ~TIME;
						 if (test_end(i, pos))//check the stop condition for synchronous and non-synchronous mode
						 { steppers[i].state = idle;
						   steppers[i].motor_pos_reached = steppers[i].starts;//signal to host
                         }
						 else if (pos == tgt && (steppers[i].flags & SYNC))//target reached but not the final destination. Condition also occurs on hard/softstop
						                                                   //hence extra condition for sync-motors 
						 { char res = runslice(i);//compute new line segment
						   //res==0 means 'master', res==1 means 'new position', res==2 means position reached (which was trapped above)
						   steppers[i].state = setpos;
						 } 
						 else if (statbuf[0] == 0) //always true
						 { //steppers[i].state = setpos;
						   steppers[i].state = status1;
						   getStatus1(i);//get ESW and error data
                         } 
					   }
					   break;
        case status1:  if (i2c_busy()) break; //status read (or other i2c activity) still busy
		               //if (statbuf[0] == 0) //the received message is complete
					   if (1)
		               { char err = statbuf[5];
					     char stall = statbuf[6] & 0x02;
					     steppers[i].esw = (statbuf[6] & 0x10) != 0;
						 statbuf[0] = 0; //release buffer
						 if (err & 0xB8) //ignore thermal warnings and steploss
						 { SETPIN11; //motor is probably in shutdown
						   steppers[i].state = idle;
						   steppers[i].pos = steppers[i].dest;
						   steppers[i].motor_pos_reached = steppers[i].starts;//signal to host
                         } 
						 else
						    steppers[i].state = setpos;
					   }
					   break;
        case stop:     if (i2c_busy()) break;
		               if (steppers[i].dest == 0)
					   { steppers[i].state = status;
					     getStatus2(i);  	  
					   }
		               else if (steppers[i].flags & (SPEED|START))
		               { steppers[i].state = setpos;
					     setPosition(i);
                         steppers[i].flags &= ~START;
					   }
					   break;
        case speed:    steppers[i].state = setpos;
		               break;
        case initmot:  if (!eeprom_busy)
		                  init_stepper(i);
					   break;
	  }
}
#endif //poll function

#endif  //AMIS30624

/***************************************************************************************************************/


#endif //USE_STEPPER


